import Redis from 'ioredis';
import dotenv from 'dotenv';

dotenv.config();

class RedisSessionManager {
    private static instance: RedisSessionManager;
    public readonly client: Redis;

    private constructor() {
        this.client = new Redis({
            host: process.env.REDIS_HOST,
            port: Number(process.env.REDIS_PORT),
            password: process.env.REDIS_PASSWORD || undefined,
            showFriendlyErrorStack: true,
        });
    }

    public static getInstance(): RedisSessionManager {
        if (!RedisSessionManager.instance) {
            RedisSessionManager.instance = new RedisSessionManager();
        }
        return RedisSessionManager.instance;
    }

    async storeObject(keyPrefix: string, objectId: string, data: Record<string, any>, expirationSec: number = 3600): Promise<void> {
        const key = `${keyPrefix}:${objectId}`;
        const dataToStore = JSON.stringify(data);
        await this.client.set(key, dataToStore, 'EX', expirationSec);
    }

    async retrieveObject(keyPrefix: string, objectId: string): Promise<Record<string, any> | null> {
        const key = `${keyPrefix}:${objectId}`;
        const data = await this.client.get(key);
        return data ? JSON.parse(data) : null;
    }

    async publishObject(channel: string, data: Record<string, any>): Promise<number> {
        const dataToPublish = JSON.stringify(data);
        return this.client.publish(channel, dataToPublish);
    }

    async shutdown(): Promise<void> {
        await this.client.quit();
    }
}

export default RedisSessionManager;
